import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;

import com.avi.Contact;



public class Simplest {
   
    static SessionFactory sessionFactory;
    
    public static void main(String[] args) {
        
        sessionFactory = 
            new Configuration().configure().buildSessionFactory();
        
        Contact example = getExampleContact();
        
        insertContact(example);
        
        readContact1(example.getId());
        readContact2(example.getId());
        readContact3(example.getId());
        readContact4(example.getId());
        readContact5(example.getId());
        readContact6(example.getId());
        readContact7(example.getId());
        readContact8(example.getId());
        readContact9(example.getId());
        
        example.setFirstName("new name");
        updateContact1(example);
        
        readContact1(example.getId());
        
        example.setFirstName("new name2");
        
        updateContact2(example);
        
        readContact2(example.getId());
        
        loadDataToDB();
        
        List c = getAllContacts();
        System.out.println(""+c.size());
        
    }
    
    /**
     * Automatic persistence
     */
    private static Contact readContact1(long id){
        Session session = sessionFactory.openSession();
        
        Contact fromDb = (Contact) session.load(Contact.class, id);
        System.out.println("readContact1() "+fromDb.getFirstName());
        session.flush();
        session.close();
        
        return fromDb;
    }
    
    /**
     * using HQL
     */
    private static Contact readContact2(long id){
        Session session = sessionFactory.openSession();
        
       Query query = session.createQuery("from Contact where id="+id);  
       
       Contact fromDb =(Contact)query.uniqueResult();
        
        
        System.out.println("readContact2() "+fromDb.getFirstName());
        session.flush();
        session.close();
        
        return fromDb;
    }
    
    /**
     * using HQL + parameter injections by name
     */
    private static Contact readContact3(long id){
        Session session = sessionFactory.openSession();
        
       Query query = session.createQuery("from Contact where id=:id");  
       query.setParameter("id", id);
       Contact fromDb =(Contact)query.uniqueResult();
        
        
        System.out.println("readContact3() "+fromDb.getFirstName());
        session.flush();
        session.close();
        
        return fromDb;
    }
    
    
    /**
     * using HQL + parameter injections place
     */
    private static Contact readContact4(long id){
        Session session = sessionFactory.openSession();
        
       Query query = session.createQuery("from Contact where id=?");  
       query.setLong(0, id);
      //also possible -  query.setParameter(0, id);
       
       Contact fromDb =(Contact)query.uniqueResult();
        
        System.out.println("readContact4() "+fromDb.getFirstName());
        session.flush();
        session.close();
        
        return fromDb;
    }
    
    /**
     * named query
     */
    private static Contact readContact5(long id){
        Session session = sessionFactory.openSession();
        
       Query query = session.getNamedQuery("getContactById_HQL");
       query.setLong(0, id);
       
       Contact fromDb =(Contact)query.uniqueResult();
        
        
        System.out.println("readContact5() "+fromDb.getFirstName());
        session.flush();
        session.close();
        
        return fromDb;
    }
    
    
    /**
     * simple native SQL
     */
    private static Contact readContact6(long id){
        Session session = sessionFactory.openSession();
        
       SQLQuery query = session.createSQLQuery("SELECT FIRSTNAME from CONTACT WHERE ID="+id);
       
       
       Object fromDb =query.uniqueResult();
        
        
        System.out.println("readContact6() "+fromDb);
        session.flush();
        session.close();
        
        return null;
    }
    
    /**
     * native SQL with scalar
     */
    private static Contact readContact7(long id){
        Session session = sessionFactory.openSession();
        
       Query query = 
           session.createSQLQuery("select firstname from Contact where id="+id)
           .addScalar("firstname", Hibernate.STRING);
      
       Object fromDb = query.uniqueResult();
        
        
        System.out.println("readContact7() "+fromDb);
        session.flush();
        session.close();
        
        return null;
    }
    
    
    /**
     * native SQL with entity
     */
    private static Contact readContact8(long id){
        Session session = sessionFactory.openSession();
        
       Query query = 
           session.createSQLQuery("select * from Contact contact where id="+id)
           .addEntity("contact",Contact.class);
      
       Contact fromDb = (Contact)query.uniqueResult();
        
        
        System.out.println("readContact8() "+fromDb.getFirstName());
        session.flush();
        session.close();
        
        return fromDb;
    }
    

    /**
     * named SQL query
     */
    private static Contact readContact9(long id){
        Session session = sessionFactory.openSession();
        
       Query query = 
           session.getNamedQuery("getContactById_SQL")
           .setLong("id", id);
      
       Contact fromDb = (Contact)query.uniqueResult();
        
        
        System.out.println("readContact9() "+fromDb.getFirstName());
        session.flush();
        session.close();
        
        return fromDb;
    }
    
    private static void insertContact(Contact contact){
       Session session = sessionFactory.openSession();
       Transaction transaction = session.beginTransaction();
       session.save(contact);
       transaction.commit();
       session.flush();
       session.close();
    }
    
    /**
     * auto
     */
    private static void updateContact1(Contact contact){
        Session session = sessionFactory.openSession();
        session.update(contact);
        session.flush();
        session.close();
     }
    
    /**
     * HSQL
     */
    private static void updateContact2(Contact contact){
        Session session = sessionFactory.openSession();
        Query query = session.createQuery("update Contact set FIRSTNAME='"+
                 contact.getFirstName()+"' where id="+contact.getId());
        query.executeUpdate();
        session.flush();
        session.close();
     }
    
    private static Contact getExampleContact(){
        Contact contact = new Contact(666,"avi", "yehuda", "aviyehuda@yahoo.com");
        return contact;
    }
    
    private static void loadDataToDB(){
        Session session = sessionFactory.openSession();
        for (int i = 0; i < 20; i++) {
            Contact contact = new Contact(i, "person "+ i , "stam", "aaa"+i+"@aaa.com");
            session.save(contact);
        }
        session.flush();
        session.close();
    }
    
    private static List getAllContacts(){
        Session session = sessionFactory.openSession();
        List res = session.createQuery("from Contact").list();
        session.flush();
        session.close();
        return res;
    }
}
